# Generated by Django 5.2.14 on 2026-06-08 12:56

from django.db import migrations, models


class Migration(migrations.Migration):

    dependencies = [
        ("standingssync", "0003_allow_empty_evewar_objects"),
    ]

    operations = [
        migrations.AddField(
            model_name="syncmanager",
            name="compress_contacts",
            field=models.BooleanField(
                default=False,
                help_text="Removes contacts which are unnecessary for calculating their effective standing.",
            ),
        ),
        migrations.AddField(
            model_name="syncmanager",
            name="exclude_character_contacts",
            field=models.BooleanField(
                default=False, help_text="Excludes all character contacts"
            ),
        ),
        migrations.AddField(
            model_name="syncmanager",
            name="exclude_negative_standings",
            field=models.BooleanField(
                default=False, help_text="Excludes all contacts with negative standing"
            ),
        ),
        migrations.AddField(
            model_name="syncmanager",
            name="exclude_positive_standings",
            field=models.BooleanField(
                default=False, help_text="Excludes all contacts with positive standing"
            ),
        ),
    ]
