# Generated by Django 5.2.14 on 2026-06-03 16:02

import django.db.models.deletion
from django.db import migrations, models


class Migration(migrations.Migration):

    dependencies = [
        ("eveuniverse", "0012_alter_evebloodline_eve_ship_type"),
        ("standingssync", "0002_improve_sync_logic"),
    ]

    operations = [
        migrations.AddField(
            model_name="evewar",
            name="last_modified",
            field=models.DateTimeField(auto_now=True),
        ),
        migrations.AlterField(
            model_name="evewar",
            name="aggressor",
            field=models.ForeignKey(
                default=None,
                null=True,
                on_delete=django.db.models.deletion.SET_NULL,
                related_name="+",
                to="eveuniverse.eveentity",
            ),
        ),
        migrations.AlterField(
            model_name="evewar",
            name="declared",
            field=models.DateTimeField(default=None, null=True),
        ),
        migrations.AlterField(
            model_name="evewar",
            name="defender",
            field=models.ForeignKey(
                default=None,
                null=True,
                on_delete=django.db.models.deletion.SET_NULL,
                related_name="+",
                to="eveuniverse.eveentity",
            ),
        ),
        migrations.AlterField(
            model_name="evewar",
            name="is_mutual",
            field=models.BooleanField(default=None, null=True),
        ),
        migrations.AlterField(
            model_name="evewar",
            name="is_open_for_allies",
            field=models.BooleanField(default=None, null=True),
        ),
    ]
