# Generated by Django 5.2.15 on 2026-06-23 12:54

from django.db import migrations, models


class Migration(migrations.Migration):

    dependencies = [
        ("standingssync", "0004_add_contact_filters_to_manager"),
    ]

    operations = [
        migrations.AddField(
            model_name="syncmanager",
            name="unmask_war_targets",
            field=models.BooleanField(
                default=False,
                help_text="Remove contacts which would mask the effective standing of war target contacts",
            ),
        ),
        migrations.AlterField(
            model_name="syncmanager",
            name="compress_contacts",
            field=models.BooleanField(
                default=False,
                help_text="Remove contacts which are unnecessary for calculating their effective standing.",
            ),
        ),
        migrations.AlterField(
            model_name="syncmanager",
            name="exclude_character_contacts",
            field=models.BooleanField(
                default=False, help_text="Exclude all character contacts"
            ),
        ),
        migrations.AlterField(
            model_name="syncmanager",
            name="exclude_negative_standings",
            field=models.BooleanField(
                default=False, help_text="Exclude all contacts with negative standing"
            ),
        ),
        migrations.AlterField(
            model_name="syncmanager",
            name="exclude_positive_standings",
            field=models.BooleanField(
                default=False, help_text="Exclude all contacts with positive standing"
            ),
        ),
    ]
