{% if has_data %}
    <img src="{{ icon_url }}" height="{{ icon_size }}" width="{{ icon_size }}">&nbsp;
    <a href="{{ profile_url }}" target="_blank">{{ name }}</a>
{% else %}
    [NO DATA]
{% endif %}
